#lang racket

(provide users
         links
         create-link
         create-user
         hash-query
         value->json)

;; just use hashes so we can go to JSON easily
;; a link is a hash: ('id: number, 'url: string, 'desc: string, 'caption: string)
;; a user is a hash: ('id: number, 'uname: string, 'passwd: string)

(define curr-link-id 0)
(define curr-user-id 0)
(define links (make-hash))
(define users (make-hash))

(define (create-link #:url url #:desc desc #:caption caption)
  (begin
    (set! curr-link-id (add1 curr-link-id))
    (let ([new-link (make-hash 
                     `((url . ,url)
                       (desc . ,desc)
                       (caption . ,caption)))])
      (hash-set! links curr-link-id new-link)
      new-link)))

(define (create-user #:uname uname #:passwd passwd)
  (begin
    (set! curr-user-id (add1 curr-user-id))
    (hash-set! users curr-user-id
               (make-hash
                `((uname . ,uname)
                  (passwd . ,passwd))))))

(define (hash-query hash key value)
  (for/or (((k v) hash))
          (if (equal? (hash-ref v key) value)
              v
              #f)))

;; good-enough json for our purposes
;; dict + list + number + string + symbol + bool -> json-string
(define (value->json v)
  (let ([arrayify 
         (λ (arr) 
           (format "[~a]" (string-join (map value->json arr) ",")))]
        [objify 
         (λ (mems) 
           (format "{~a}" (string-join mems ",")))]
        [quotify (λ (str) (format "\"~a\"" str))])
    (match v
      [(? dict? v)
       (objify
        (flatten
         (dict-map 
          v
          (λ (k v) (if (equal? k '_id)
                       empty
                       (string-append* 
                        `(,(quotify (symbol->string k))
                          ":"
                          ,(value->json v))))))))]
      [(? list? v) (arrayify v)]
      [(? number? v) (number->string v)]
      [(? string? v) (quotify v)]
      [(? symbol? v)
       (quotify (symbol->string v))]
      [(? (λ (v) (equal? 'null v)) v) "null"]
      [#t "true"]
      [#f "false"])))


;; data

((λ ()
  (create-link #:url "http://www.google.com" 
               #:desc "Pretty good search engine... check it out"
               #:caption "")
  
  (create-link #:url "http://www.yahoo.com" #:desc "These guys have some neat ideas, too.  Their search is not quite as good as those Google guys, but their community is just excellent and I hear they have the guy who designed ADsafe, too."  #:caption "")
  
  (create-link #:url "http://www.dailyshow.com"
               #:desc "Hilarious stuff here"
               #:caption "")
  
  (create-user #:uname "poster"
               #:passwd "poster")))
